/*	Math functions for Csound coded by Paris Smaragdis 1994         */
/*	Berklee College of Music Csound development team                */

#include "cs.h"
#include "cmath.h"
#include <math.h>

#ifndef RAND_MAX
#define RAND_MAX        (32767)
#endif

void ipow(POW *p)		/*      Power for i-rate */
{
    *p->sr = pow( *p->in, *p->pow);	/*      sophisticated code!     */
}

void apow(POW *p)		/*      Power routine for a-rate  */
{
    register long n = ksmps;
    register float *in = p->in, *out = p->sr;

    do {
	*out++ = pow( *in++, *p->pow) / *p->norm;
    } while( --n);
}

void kpow(POW *p)		/*      Power routine for k-rate        */
{
    *p->sr = pow( *p->in, *p->pow) / *p->norm;
}

void seedrand(PRAND *p)
{
    printf("Seeding with %.3f\n", *p->out);
    srand( ( unsigned int)*p->out);
}

void alinear(PRAND *p)		/* Linear random functions      */
{
    register long n = ksmps;
    register float *out = p->out;

    do {
	*out++ = linrand( *p->arg1);
    } while( --n);
}

void iklinear(PRAND *p)
{
    register float *out = p->out;
    *out++ = linrand( *p->arg1);
}

void atrian(PRAND *p)		/*      Triangle random functions       */
{
    register long n = ksmps;
    register float *out = p->out;

    do {
	*out++ = trirand( *p->arg1);
    } while( --n);
}

void iktrian(PRAND *p)
{
    register float *out = p->out;
    *out++ = trirand( *p->arg1);
}

void aexp(PRAND *p)		/*      Exponential random functions    */
{
    register long n = ksmps;
    register float *out = p->out;

    do {
	*out++ = exprand( *p->arg1);
    } while( --n);
}

void ikexp(PRAND *p)
{
    register float *out = p->out;
    *out++ = exprand( *p->arg1);
}

void abiexp(PRAND *p)		/*      Bilateral exponential random functions  */
{
    register long n = ksmps;
    register float *out = p->out;

    do {
	*out++ = biexprand( *p->arg1);
    } while( --n);
}

void ikbiexp(PRAND *p)
{
    register float *out = p->out;
    *out++ = biexprand( *p->arg1);
}

void agaus(PRAND *p)		/*      Gaussian random functions       */
{
    register long n = ksmps;
    register float *out = p->out;

    do {
	*out++ = gaussrand( *p->arg1);
    } while( --n);
}

void ikgaus(PRAND *p)
{
    register float *out = p->out;
    *out++ = gaussrand( *p->arg1);
}

void acauchy(PRAND *p)		/*      Cauchy random functions */
{
    register long n = ksmps;
    register float *out = p->out;

    do {
	*out++ = cauchrand( *p->arg1);
    } while( --n);
}

void ikcauchy(PRAND *p)
{
    register float *out = p->out;
    *out++ = cauchrand( *p->arg1);
}

void apcauchy(PRAND *p)		/*      Positive Cauchy random functions        */
{
    register long n = ksmps;
    register float *out = p->out;

    do {
	*out++ = pcauchrand( *p->arg1);
    } while( --n);
}

void ikpcauchy(PRAND *p)
{
    register float *out = p->out;
    *out++ = pcauchrand( *p->arg1);
}

void abeta(PRAND *p)		/*      Beta random functions   */
{
    register long n = ksmps;
    register float *out = p->out;

    do {
	*out++ = betarand( *p->arg1, *p->arg2, *p->arg3);
    } while( --n);
}

void ikbeta(PRAND *p)
{
    register float *out = p->out;
    *out++ = betarand( *p->arg1, *p->arg2, *p->arg3);
}

void aweib(PRAND *p)		/*      Weibull randon functions        */
{
    register long n = ksmps;
    register float *out = p->out;

    do {
	*out++ = weibrand( *p->arg1, *p->arg2);
    } while( --n);
}

void ikweib(PRAND *p)
{
    register float *out = p->out;
    *out++ = weibrand( *p->arg1, *p->arg2);
}

void apoiss(PRAND *p)		/*      Poisson random funcions */
{
    register long n = ksmps;
    register float *out = p->out;

    do {
	*out++ = poissrand( *p->arg1);
    } while( --n);
}

void ikpoiss(PRAND *p)
{
    register float *out = p->out;
    *out++ = poissrand( *p->arg1);
}


/* * * * * * RANDOM NUMBER GENERATORS * * * * * */


#define unirand()	(float)((double)rand()/(double)RAND_MAX)

float unifrand(void)
{
    return unirand();
}

float linrand(float range)	/*      linear distribution routine     */
{
    register float r1, r2;

    r1 = unirand();
    r2 = unirand();

    if( r1 > r2)
	r1 = r2;

    return( r1 * range);
}

float trirand(float range)	/*      triangle distribution routine   */
{
    register float r1, r2;

    r1 = unirand();
    r2 = unirand();

    return( (( r1 + r2) - 1) * range);
}
		
float exprand(float l)		/*      exponential distribution routine        */
{
    register float r1;

    if( l < 0) return( 0);

    do {
	r1 = unirand();
    } while( r1 == 0);

    return( -log( r1) * l/10);
}

float biexprand(float l)	/* bilateral exponential distribution routine      */
{
    register float r1;

    if( l < 0) return ( 0);

    do {
	r1 = 2 * unirand();
    } while( r1 == 0 || r1 == 2); 

    if( r1 > 1)     {
	r1 = 2.0 - r1;
	return( -log(r1) * l/10);
    }
    return( log(r1) * l/10);
}

float gaussrand(float s)	/*      gaussian distribution routine   */
{
    register float r1 = 0;
    register int n = 12;
    s /= 3.83;

    do {
	r1 += unirand();
    } while( --n);

    return( s * (r1 - 6));
}

float cauchrand(float a)	/*      cauchy distribution routine     */
{
    register float r1;
    a /= 318.3;

    do {
	r1 = unirand();
    } while( r1 == .5);

    r1 *= pi;
    return( a * tan( r1));
}

float pcauchrand(float a)	/*      positive cauchy distribution routine    */
{
    register float r1;
    a /= 318.3;

    do {
	r1 = unirand();
    } while( r1 == 1);

    r1 *= pi / 2;
    return( a * tan( r1));
}

float betarand(float range, float a, float b) /*      beta distribution routine       */
{
    register float r1, r2, r3;

    if( a < 0 || b < 0 ) return( 0); 

    do {
	do {
	    r1 = unirand();
	} while( r1 == 0);
	
	do {
	    r2 = unirand();
	} while( r2 == 0);
	
	r3 = r1 = pow(r1, 1.0 / a);
	r2 = pow(r2, 1.0 / b);
    } while( (r1 + r2) > 1.0);

    return( (r3 / (r1 +r2)) * range);
}

float weibrand(float s, float t) /*      weibull distribution routine    */
{
    register float r1, r2;

    if( t < 0 ) return( 0);

    do {
	r1 = unirand();
    } while( r1 == 0 || r1 == 1);

    r2 = 1.0 / ( 1.0 - r1);

    return( s * pow( log( r2), ( 1.0 / t)));
}

float poissrand(float l)	/*      poisson distribution routine    */
{
    register float r1, r2, r3;

    if( l < 0 ) return( 0);

    r1 = unirand();
    r2 = exp( -l);
    r3 = 0;

    while( r1 >= r2){
	r3++;
	r1 *= unirand();
    }

    return( r3);
}
